#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "abcdisk.h"

#define F(c,h,s) (c),(h),(s),((c)*(h)*(s))

static const struct diskfmt formats[] = {
    { "fd2",	    F( 40,1, 8), 0,  6, { 8, 16 },  8 },
    { "dd80",	    F( 40,1, 8), 0,  6, { 8, 16 },  8 },
    { "fd2d",	    F( 40,1,16), 0,  6, { 8, 16 },  8 },
    { "dd82",	    F( 40,1,16), 0,  6, { 8, 16 },  8 },
    { "abc830",     F( 40,1,16), 0,  6, { 8, 16 },  8 },
    { "mo",         F( 40,1,16), 0,  6, { 8, 16 },  8 },
    { "abc830-ufd", F( 40,1,16), 0,  6, { 16, 0 }, 16 },
    { "fd4d",       F( 40,2,16), 0,  6, { 8, 16 },  8 },
    { "dd84",       F( 40,2,16), 0,  6, { 8, 16 },  8 },
    { "abc832",     F( 80,2,16), 2, 14, { 16, 0 }, 16 },
    { "abc832-ufd", F( 80,2,16), 2, 14, { 8, 16 },  8 },
    { "abc834",     F( 80,2,16), 2, 14, { 16, 0 }, 16 },
    { "mf",         F( 80,2,16), 2, 14, { 16, 0 }, 16 },
    /*
     * 8" floppies have a different logical sector/track
     * mapping than the physical one (which is 77/2/26), and
     * then the last virtual track is only partially filled.
     */
    { "abc838",     126,2,16, 4004, 0,  6, { 8, 16 },  8 },
    { "abc838-ufd", 126,2,16, 4004, 2, 14, { 16, 0 }, 16 },
    { "sf",         126,2,16, 4004, 2, 14, { 16, 0 }, 16 },
    { "hd",         F(238,8,32), 5, 14, { 16, 0 }, 16 },
};

static const int nformats = (sizeof(formats)/sizeof(formats[0]));

const struct diskfmt *get_format(const char *name)
{
    const struct diskfmt *fmt;
    int i;

    for (i = 0, fmt = formats; i < nformats; i++, fmt++) {
	if (!strcmp(fmt->name, name))
	    return fmt;
    }

    return NULL;
}

void show_formats(FILE *f)
{
    const struct diskfmt *fmt;
    int i;

    fprintf(f, "Format      Tracks  Sides  Sec/trk  Cluster  Sectors  Total  System   MFD\n");

    for (i = 0, fmt = formats; i < nformats; i++, fmt++) {
	fprintf(f, "%-11s %4d %6d %7d %9d %9d %6dK %7d %3d/%2d\n",
		fmt->name, fmt->c, fmt->h, fmt->s,
		1 << fmt->cluster_shift,
		fmt->sectors, fmt->sectors >> 2, fmt->syssec,
		fmt->dirsec[0], fmt->dirsecs);
    }
}
