/*
 * mangle.c:
 *
 * Convert a filename in host encoding to a filename in ABC format,
 * without dot.
 *
 * Returns 0 for OK, -1 for failure (filename cannot be converted)
 */

#include "abcdisk.h"

int mangle_name(unsigned char *dst, const char *src)
{
    static const wchar_t srcset[] =
        L"0123456789_."
        L"ÉABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÅÜÆØ"
        L"éabcdefghijklmnopqrstuvwxyzäöåüæø";
    static const unsigned char dstset[] =
        "0123456789_."
        "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^[\\"
        "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^[\\";
    unsigned char *d = dst;
    const char *s, *p;
    wchar_t sc;
    const wchar_t *scp;
    char dc;
    int n;

    if ((p = strrchr(src, '/')))
        s = p + 1;
    else
        s = src;

    memset(dst, ' ', 11);
    dst[11] = '\0';

    mbtowc(NULL, NULL, 0);      /* Reset the shift state */

    while (d < dst + 11 && (n = mbtowc(&sc, s, (size_t) ~ 0)) > 0) {
        s += n;

        if ((scp = wcschr(srcset, sc))) {
            dc = dstset[scp - srcset];
        } else {
            dc = '_';
        }

        if (dc == '.')
            d = dst + 8;
        else
            *d++ = dc;
    }

    return 0;
}
